"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Transactional = void 0;
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const typeorm_als_constants_1 = require("./typeorm-als.constants");
const typeorm_als_enums_1 = require("./typeorm-als.enums");
const typeorm_als_utils_1 = require("./typeorm-als.utils");
function Transactional(options) {
    const opts = Object.assign(Object.assign({}, typeorm_als_constants_1.DEFAULT_OPTIONS), options);
    const { propagation, isolation } = opts;
    const injectStorage = (0, common_1.Inject)(typeorm_als_constants_1.ASYNC_STORAGE);
    const injectDataSource = (0, common_1.Inject)((0, typeorm_1.getDataSourceToken)(opts.dataSource));
    return (target, propertyKey, propertyDescriptor) => {
        injectStorage(target, 'storage');
        injectDataSource(target, 'dataSource');
        const originalMethod = propertyDescriptor.value;
        propertyDescriptor.value = function (...args) {
            return __awaiter(this, void 0, void 0, function* () {
                const { dataSource, storage } = this;
                const runOriginal = () => originalMethod.apply(this, args);
                const runWithTransaction = () => __awaiter(this, void 0, void 0, function* () {
                    const store = storage.getStore();
                    const runWithNewTransaction = () => {
                        const transactionCallback = (manager) => __awaiter(this, void 0, void 0, function* () {
                            (0, typeorm_als_utils_1.setEntityManager)(store, opts.dataSource, manager);
                            try {
                                return yield runOriginal();
                            }
                            finally {
                                (0, typeorm_als_utils_1.deleteEntityManager)(store, opts.dataSource);
                            }
                        });
                        if (isolation) {
                            return dataSource.transaction(isolation, transactionCallback);
                        }
                        else {
                            return dataSource.transaction(transactionCallback);
                        }
                    };
                    const entityManager = (0, typeorm_als_utils_1.getEntityManager)(store, opts.dataSource);
                    switch (propagation) {
                        case typeorm_als_enums_1.Propagation.MANDATORY:
                            if (!entityManager) {
                                throw new Error("No existing transaction found for transaction marked with propagation 'MANDATORY'");
                            }
                            return runOriginal();
                        case typeorm_als_enums_1.Propagation.NESTED:
                            return runWithNewTransaction();
                        case typeorm_als_enums_1.Propagation.NEVER:
                            if (entityManager) {
                                throw new Error("Found an existing transaction, transaction marked with propagation 'NEVER'");
                            }
                            return runOriginal();
                        case typeorm_als_enums_1.Propagation.NOT_SUPPORTED:
                            if (entityManager) {
                                (0, typeorm_als_utils_1.deleteEntityManager)(store, opts.dataSource);
                                const result = yield runOriginal();
                                (0, typeorm_als_utils_1.setEntityManager)(store, opts.dataSource, entityManager);
                                return result;
                            }
                            return runOriginal();
                        case typeorm_als_enums_1.Propagation.REQUIRED: {
                            if (entityManager) {
                                return runOriginal();
                            }
                            return runWithNewTransaction();
                        }
                        case typeorm_als_enums_1.Propagation.REQUIRES_NEW:
                            return runWithNewTransaction();
                        case typeorm_als_enums_1.Propagation.SUPPORTS:
                            return runOriginal();
                    }
                });
                if (!storage.getStore()) {
                    return storage.run(new Map(), () => {
                        return runWithTransaction();
                    });
                }
                return runWithTransaction();
            });
        };
        Reflect.getMetadataKeys(originalMethod).forEach((previousMetadataKey) => {
            const previousMetadata = Reflect.getMetadata(previousMetadataKey, originalMethod);
            Reflect.defineMetadata(previousMetadataKey, previousMetadata, propertyDescriptor.value);
        });
        Object.defineProperty(propertyDescriptor.value, 'name', {
            value: originalMethod.name,
            writable: false,
        });
    };
}
exports.Transactional = Transactional;
